-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsGivers", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LDQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-DailyQuests")
local LFQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-ForbidenQuests")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local select, pairs, format, unpack = select, pairs, string.format, unpack
local GetGossipAvailableQuests, GetGossipActiveQuests = GetGossipAvailableQuests, GetGossipActiveQuests
local GetNumActiveQuests, GetNumAvailableQuests = GetNumActiveQuests, GetNumAvailableQuests
local GetActiveTitle, GetActiveLevel, IsActiveQuestTrivial = GetActiveTitle, GetActiveLevel, IsActiveQuestTrivial
local GetAvailableTitle, GetAvailableLevel, IsAvailableQuestTrivial = GetAvailableTitle, GetAvailableLevel, IsAvailableQuestTrivial
local AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest = AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest

-----------------
--- variables ---
-----------------
local trivialText = "|cff%02x%02x%02x[%s%s]|r "..TRIVIAL_QUEST_DISPLAY
local normalText = "|cff%02x%02x%02x[%s%s]|r "..NORMAL_QUEST_DISPLAY

module.dailies = {}
module.forbiddenQuests = {}
module.texCoords = {
	available = { 0.125, 0.25, 0.5, 1 },
	active = { 0.25, 0.375, 0.5, 1 },
	availabledaily = { 0.375, 0.5, 0.5, 1 },
	activedaily = { 0.5, 0.625, 0.5, 1 },
}

-----------------------
--- Tools functions ---
-----------------------
local function getDifficultyColor(levelDiff)
	if levelDiff >= 5 then
		return QuestDifficultyColors["impossible"]
	elseif levelDiff >= 3 then
		return QuestDifficultyColors["verydifficult"]
	elseif levelDiff >= -2 then
		return QuestDifficultyColors["difficult"]
	elseif -levelDiff <= GetQuestGreenRange() then
		return QuestDifficultyColors["standard"]
	else
		return QuestDifficultyColors["trivial"]
	end
end

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		addQuestLevel = true,
		fadeNotCompletedQuests = true,
		automaticAccept = 4,
		automaticAcceptEscort = false,
		automaticTurnIn = false,
	}
}

function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsGivers", defaults, "Default")
	self:SetEnabledState(core:getModuleEnabled("questsGivers"))
	core.moduleState["questsGivers"] = core:getModuleEnabled("questsGivers")
end

function module:OnEnable()
	for _, v in pairs(LDQ) do
		self.dailies[v] = true
	end
	for _, v in pairs(LFQ) do
		self.forbiddenQuests[v] = true
	end
	core.del(LDQ)
	core.del(LFQ)
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM")
end

function module:clickQuest(button)
	if button:IsVisible() then
		local title = button.title
		if button.type == "Available" and (self.db.profile.automaticAccept == 1 or (self.db.profile.automaticAccept == 2 and self.dailies[title]) or (self.db.profile.automaticAccept == 3 and not self.dailies[title])) then
			button:Click()
		elseif button.type == "Active" and self.db.profile.automaticTurnIn and ((core.questsTitleToUid[title] and (core.quests[core.questsTitleToUid[title]].complete[core.playerName] == 1 or core.quests[core.questsTitleToUid[title]].numObjectives == 0)) or (not core.questsTitleToUid[title])) then
			button:Click()
		end
	end
end

function module:loopAvailableQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	for i = 1, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		local title, level, trivial = select(i, ...)
		local buttonName = "GossipTitleButton"..self.buttonIndex
		local titleButton = _G[buttonName]
		titleButton.title = title
		local questIconButton = _G[buttonName.."GossipIcon"]
		questIconButton:SetTexture("Interface\\MiniMap\\ObjectIcons")
		questIconButton:SetTexCoord(unpack(self.texCoords["available"..(self.dailies[title] and "daily" or "")]))
		questIconButton:SetVertexColor(1, 1, 1, 1)
		if self.db.profile.addQuestLevel then
			local color = getDifficultyColor(level - core.playerLevel)
			titleButton:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, self.dailies[title] and core.db.profile.quests.daily or "", title)
			titleButton:SetHeight(titleButton:GetTextHeight() + 2)
		end
	end
	self.buttonIndex = self.buttonIndex + 1
end

function module:loopActiveQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	for i = 1, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		local title, level, trivial = select(i, ...)
		local buttonName = "GossipTitleButton"..self.buttonIndex
		local titleButton = _G[buttonName]
		titleButton.title = title
		local questIconButton = _G[buttonName.."GossipIcon"]
		questIconButton:SetTexture("Interface\\MiniMap\\ObjectIcons")
		questIconButton:SetTexCoord(unpack(self.texCoords["active"..(self.dailies[title] and "daily" or "")]))
		if self.db.profile.fadeNotCompletedQuests then
			local uid = core.questsTitleToUid[title]
			if uid and core.quests[uid].playerComplete == 0 then
				questIconButton:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			else
				questIconButton:SetVertexColor(1, 1, 1, 1)
			end
		else
			questIconButton:SetVertexColor(1, 1, 1, 1)
		end
		if self.db.profile.addQuestLevel then
			local color = getDifficultyColor(level - core.playerLevel)
			titleButton:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, self.dailies[title] and core.db.profile.quests.daily or "", title)
			titleButton:SetHeight(titleButton:GetTextHeight() + 2)
		end
	end
end

function module:loopOptions(...)
	local optionIndex = self.buttonIndex
	for i=1, select("#", ...), 2 do
		optionIndex = optionIndex + 1
		local titleButtonIcon = _G["GossipTitleButton"..optionIndex.."GossipIcon"]
		titleButtonIcon:SetTexCoord(0, 1, 0, 1)
	end
end

function module:GOSSIP_SHOW()
	self.buttonIndex = 0
	self:loopAvailableQuests(GetGossipAvailableQuests())
	self:loopActiveQuests(GetGossipActiveQuests())
	self:loopOptions(GetGossipOptions())
	if IsShiftKeyDown() then return end
	for i = 1, self.buttonIndex do
		self:clickQuest(_G["GossipTitleButton" .. i])
	end
end

function module:QUEST_GREETING()
	local numActive, numAvailable = GetNumActiveQuests() or 0, GetNumAvailableQuests() or 0
	self.buttonIndex = 0
	for i = 1, numActive do
		self.buttonIndex = self.buttonIndex + 1
		local title = GetActiveTitle(i)
		local buttonName = "QuestTitleButton"..self.buttonIndex
		local titleButton = _G[buttonName]
		titleButton.title = title
		local questIconButton = _G[buttonName.."QuestIcon"]
		questIconButton:SetTexture("Interface\\MiniMap\\ObjectIcons")
		questIconButton:SetTexCoord(unpack(self.texCoords["active"..(self.dailies[title] and "daily" or "")]))
		if self.db.profile.fadeNotCompletedQuests then
			local uid = core.questsTitleToUid[title]
			if uid and core.quests[uid].playerComplete == 0 then
				questIconButton:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			else
				questIconButton:SetVertexColor(1, 1, 1, 1)
			end
		else
			questIconButton:SetVertexColor(1, 1, 1, 1)
		end
		if self.db.profile.addQuestLevel then
			local level = GetActiveLevel(i)
			local color = getDifficultyColor(level - core.playerLevel)
			local trivial = IsActiveQuestTrivial(i)
			titleButton:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, self.dailies[title] and core.db.profile.quests.daily or "", title)
			titleButton:SetHeight(titleButton:GetTextHeight() + 2)
		end
	end
	for i = 1, numAvailable do
		self.buttonIndex = self.buttonIndex + 1
		local title = GetAvailableTitle(i)
		local buttonName = "QuestTitleButton"..self.buttonIndex
		local titleButton = _G[buttonName]
		titleButton.title = title
		local questIconButton = _G[buttonName.."QuestIcon"]
		questIconButton:SetTexture("Interface\\MiniMap\\ObjectIcons")
		questIconButton:SetTexCoord(unpack(self.texCoords["available"..(self.dailies[title] and "daily" or "")]))
		questIconButton:SetVertexColor(1, 1, 1, 1)
		if self.db.profile.addQuestLevel then
			local level = GetAvailableLevel(i)
			local color = getDifficultyColor(level - core.playerLevel)
			local trivial = IsAvailableQuestTrivial(i)
			titleButton:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, self.dailies[title] and core.db.profile.quests.daily or "", title)
			titleButton:SetHeight(titleButton:GetTextHeight() + 2)
		end
	end
	if IsShiftKeyDown() then return end
	for i = 1, self.buttonIndex do
		self:clickQuest(_G['QuestTitleButton' .. i])
	end
end

function module:QUEST_DETAIL()
	local title = GetTitleText()
	if not IsShiftKeyDown() and (self.db.profile.automaticAccept == 1 or (self.db.profile.automaticAccept == 2 and self.dailies[title]) or (self.db.profile.automaticAccept == 3 and not self.dailies[title])) and not self.forbiddenQuests[title] then
		AcceptQuest()
	end
end

function module:QUEST_COMPLETE(event)
	if not IsShiftKeyDown() and self.db.profile.automaticTurnIn and GetNumQuestChoices() <= 1 then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function module:QUEST_PROGRESS()
	if not IsShiftKeyDown() and self.db.profile.automaticTurnIn and IsQuestCompletable() then
		CompleteQuest()
	end
end

function module:QUEST_ACCEPT_CONFIRM()
	if not IsShiftKeyDown() and self.db.profile.automaticAcceptEscort then
		ConfirmAcceptQuest()
	end
end